#define INPUT_MAX_LEN 100
#include "configure.h"

      program chopdens
      implicit none
c
#include "common_main.h"
#include "common_dust.h"
c
c     Read all the data
c
      call read_all()
c
c     Do the chopping
c
      call do_chopping()
c
c     Write the new density
c
      call write_new_dens()
c
      end



c     --------------------------------------------------------------
c                     READ THE GRID AND SETUP
c     --------------------------------------------------------------
      subroutine read_all()
      implicit none
      doubleprecision dum
      integer i,l,ir,it,k,j,inu,imirt,nr,nt,iformat,nspec,ispec
      logical fex,ierror
      character*160 string
c
#include "common_main.h"
#include "common_dust.h"
c
c     Check whether the chopdens.inp file exists
c
      inquire(file='chopdens.inp',exist=fex)
      if(.not.fex) then
          write(*,*) 'Have not chopped the density, since chopdens.inp',
     %      ' does not exist.'
          stop
      endif
      open(unit=1,file='chopdens.inp')
      read(1,'(A80)') string
      close(1)
      l=len_trim(string)
      if(l.eq.0) then
          string='a'
          l=1
      endif
      i=0
 10   i=i+1
      if(i.ge.l) goto 15
      if(string(i:i).eq.' ') goto 10
 15   continue
      if((string(i:i).eq.'1').or.(string(i:i).eq.'2').or.
     %   (string(i:i).eq.'3').or.(string(i:i).eq.'4').or.
     %   (string(i:i).eq.'5').or.(string(i:i).eq.'6').or.
     %   (string(i:i).eq.'7').or.(string(i:i).eq.'8').or.
     %   (string(i:i).eq.'9')) then 
c
c         Old formatted style input 
c
          open(unit=1,file='chopdens.inp',status='old')
          read(1,*) iformat
          read(1,*) chop_taumax
          read(1,*) chop_lambda
          read(1,*) chop_smoothindex
          close(1)
      else
c
c         New namelist style input
c
c         ...Defaults
c
          chop_taumax = 1d6
          chop_lambda = 0.55
          chop_smoothindex = 1.d0
c
c         ...Read
c
          open(unit=1,file='chopdens.inp')
          call read_input_file()
          close(1)
          call parse_input_double('taumax@',chop_taumax)
          call parse_input_double('lambda@',chop_lambda)
          call parse_input_double('smooth@',chop_smoothindex)
          call check_all_lines_ok(ierror)
          if(ierror) stop
      endif
c
c     Read the R grid
c     
      open(unit=3,file='radius.inp',status='old')
      read(3,*) grid_nr
      if(grid_nr.gt.FRSIZE_R) then
          write(*,*) 'ERROR: radius.inp has more radial gridpoints'
          write(*,*) '       than FRSIZE_R. '
          write(*,*) '       Recompile with larger FRSIZE_R'
          stop 13
      endif
      do ir=1,grid_nr
          read(3,*) grid_r(ir)
      enddo
      close(3)
c
c     Read the Theta grid
c
      open(unit=3,file='theta.inp',status='old')
      read(3,*) grid_nt,imirt
      if(imirt.ne.1) stop 982
      if(grid_nt.gt.FRSIZE_THETA) then
          write(*,*) 'ERROR: theta.inp has more gridpoints'
          write(*,*) '       than FRSIZE_THETA. '
          write(*,*) '       Recompile with larger FRSIZE_THETA'
          stop 13
      endif
      do it=1,grid_nt
          read(3,*) grid_theta(it)
      enddo
      close(3)
c
c     Read the dust density
c
      open(unit=1,file='dustdens.inp',status='old')
      read(1,*) nspec,nr,nt,j
      if((nr.ne.grid_nr).or.(nt.ne.grid_nt)) then
          write(*,*) nr,nt,grid_nr,grid_nt
          stop 721
      endif
      do ispec=1,nspec
          do ir=1,nr
              do it=1,nt
                  read(1,*) dum
                  if(dum.gt.1d-90) then
                      dust_dens_orig(it,ir,ispec) = dum
                  else
                      dust_dens_orig(it,ir,ispec) = 1d-90
                  endif
              enddo
          enddo
      enddo
      close(1)
c
c     Read the frequency
c     
      open(unit=1,file='frequency.inp',status='old')
      read(1,*) freq_nr
      if(freq_nr.gt.FRSIZE_FREQ) then
          write(*,*) 'ERROR: frequency.inp has more gridpoints'
          write(*,*) '       than FRSIZE_FREQ. '
          write(*,*) '       Recompile with larger FRSIZE_FREQ'
          stop 13
      endif          
      do inu=1,freq_nr
          read(1,*) freq_nu(inu)
      enddo
      close(1)
c
c     Read the dust opacities
c
      call read_dustdata()
c
c     Do two checks
c
      if(nspec.ne.dust_nr_species) then
          write(*,*) 'ERROR: Nr of dust opacities given is not equal'
          write(*,*) '       to the nr of dust species in the density'
          write(*,*) '       setup.' 
          stop 
      endif
      do ispec=1,dust_nr_species
          if(dust_nr_size(ispec).ne.1) then
              write(*,*) 'ERROR: No size distributions ',
     %             'allowed in chopdens.' 
              stop 
          endif
      enddo
c
c     Done reading...
c
      end


c     --------------------------------------------------------------
c                        WRITE THE NEW DENSITY 
c     --------------------------------------------------------------
      subroutine write_new_dens()
      implicit none
c
#include "common_main.h"
#include "common_dust.h"
c
      integer ir,it,ispec
c
      open(unit=1,file='dustdens.inp',status='unknown')
      write(1,*) dust_nr_species,grid_nr,grid_nt,1
      do ispec=1,dust_nr_species
          do ir=1,grid_nr
              do it=1,grid_nt
                  write(1,*) dust_dens_new(it,ir,ispec)
              enddo
          enddo
      enddo
      close(1)
      
c
      end


c     --------------------------------------------------------------
c                          DO THE CHOPPING
c     --------------------------------------------------------------
      subroutine do_chopping()
      implicit none
c
#include "common_main.h"
#include "common_dust.h"
c
      common/ctcom/ct_kappa,ct_pl,ct_taumax,ct_z,ct_rho_orig,ct_rho_new
      doubleprecision ct_kappa(FRSIZE_THETA),ct_pl,ct_taumax
      doubleprecision ct_z(FRSIZE_THETA)
      doubleprecision ct_rho_orig(FRSIZE_THETA)
      doubleprecision ct_rho_new(FRSIZE_THETA)
      common/nctcom/ct_nz
      integer ct_nz
c
      doubleprecision nu,eps,find_dust_kappa,kap0,kap1,temp
      doubleprecision kappa(DUST_SPECIES_MAX)
      doubleprecision z(FRSIZE_THETA),alpha(FRSIZE_THETA)
      doubleprecision rho_orig(FRSIZE_THETA),rho_new(FRSIZE_THETA)
      doubleprecision rho(FRSIZE_THETA)
      doubleprecision rho0,rho1,lrho0,lrho1
      doubleprecision pi,integrate,tautot,dummy,ct_routine
      doubleprecision logrhomax,zbrent,chopacc
      doubleprecision kappa_av(FRSIZE_THETA),kapav(FRSIZE_THETA)
      parameter(pi=3.1415926535897932385d0)
      integer ir,it,inu,iz,ispec
c
      external ct_routine
c
c     Define the chopping accuracy
c
      chopacc = 0.01d0
c
c     First find the kappa at the wavelength chop_lambda
c
      temp  = 100.d0 ! Dummy temperature
      nu    = 2.9979d14/chop_lambda
      call hunt(freq_nu,freq_nr,nu,inu)
      if((inu.le.0).or.(inu.ge.freq_nr)) then
          write(*,*) 'ERROR: Lambda out of range...'
          stop 
      endif
      eps   = ( nu - freq_nu(inu) ) / ( freq_nu(inu+1) - freq_nu(inu) )
      do ispec=1,dust_nr_species
          kap0         = find_dust_kappa(inu,1,ispec,temp,1,1)
          kap1         = find_dust_kappa(inu+1,1,ispec,temp,1,1)
          kappa(ispec) = (1.d0-eps)*kap0 + eps*kap1
      enddo
c
c     Do a loop over radius
c
      do ir=1,grid_nr
c
c         First copy the old density into the new density
c
          do ispec=1,dust_nr_species
              do it=1,grid_nt
                  dust_dens_new(it,ir,ispec) = 
     %                   dust_dens_orig(it,ir,ispec)
              enddo
          enddo
c
c         Compute the total dust density (sum over species)
c
          do it=1,grid_nt
              rho_orig(it) = 0.d0
              do ispec=1,dust_nr_species
                  rho_orig(it) = rho_orig(it) + 
     %                   dust_dens_orig(it,ir,ispec)
              enddo
          enddo
c
c         Compute the average kappa at each z
c
          do it=1,grid_nr
              kappa_av(it) = 0.d0
              do ispec=1,dust_nr_species
                  kappa_av(it) = kappa_av(it) + 
     %               dust_dens_orig(it,ir,ispec) * kappa(ispec)
              enddo
              kappa_av(it) = kappa_av(it) / rho_orig(it)
          enddo
c
c         Find the total vertical optical depth 
c
          do it=grid_nt,1,-1
              z(grid_nt-it+1)     = grid_r(ir) * 
     %                              ( pi/2 - grid_theta(it) )
              rho(grid_nt-it+1)   = rho_orig(it)
              alpha(grid_nt-it+1) = rho_orig(it) * kappa_av(it)
              kapav(grid_nt-it+1) = kappa_av(it)
          enddo
          tautot = integrate(grid_nt,z,alpha)
c
c         If this exceeds the maximum, then we must limit this
c
          if(tautot.gt.chop_taumax) then
c
c             Put all the relevant data in a common block
c
              ct_pl               = -abs(chop_smoothindex)
              ct_nz               = grid_nt
              ct_taumax           = chop_taumax
              do iz=1,grid_nt
                  ct_z(iz)        = z(iz) 
                  ct_rho_orig(iz) = rho(iz)
                  ct_kappa(iz)    = kapav(iz)
              enddo
c
c             Find minimum and maximum rho
c
              rho0      = 1.d99
              rho1      = 0.d0
              do iz=1,grid_nt
                  if(ct_rho_orig(iz).gt.rho1) rho1=ct_rho_orig(iz)
                  if(ct_rho_orig(iz).lt.rho0) rho0=ct_rho_orig(iz)
              enddo
c
c             Call zbrent to find the correct rhomax
c
              lrho0     = log(rho0)
              lrho1     = log(rho1*1d6)
              logrhomax = zbrent(ct_routine,lrho0,lrho1,chopacc)
c
c             Put the density a tiny bit lower than this, to make
c             sure that it won't be chopper again if this routine is
c             called again
c
              logrhomax = logrhomax - 2*chopacc
c
c             Call the chopper routine once more with logrhomax
c             so make sure that we have the correct rho_new
c
              dummy = ct_routine(logrhomax)
c
c             Now copy eveything to rho_new
c
              do it=grid_nt,1,-1
                  rho_new(it) = ct_rho_new(grid_nt-it+1)
              enddo
c     
c             Compute the new dust_rho by scaling the old one with
c             the ratio rho_new/rho_old
c
              do it=1,grid_nt
                  do ispec=1,dust_nr_species
                      dust_dens_new(it,ir,ispec) = 
     %                     dust_dens_orig(it,ir,ispec) *
     %                     (rho_new(it)/rho_orig(it))
                  enddo
              enddo
          endif
c
c         New radius...
c
      enddo
c
      end



c     --------------------------------------------------------------
c                   CHOPPER ROUTINE FOR THE ZBRENT
c     --------------------------------------------------------------
      function ct_routine(logrhomax)
      implicit none
      doubleprecision logrhomax,ct_routine
c
      common/ctcom/ct_kappa,ct_pl,ct_taumax,ct_z,ct_rho_orig,ct_rho_new
      doubleprecision ct_kappa(FRSIZE_THETA),ct_pl,ct_taumax
      doubleprecision ct_z(FRSIZE_THETA)
      doubleprecision ct_rho_orig(FRSIZE_THETA)
      doubleprecision ct_rho_new(FRSIZE_THETA)
      common/nctcom/ct_nz
      integer ct_nz
c
      doubleprecision tautot,rhomax,integrate,alpha(FRSIZE_THETA)
      integer iz
c
c     Find the rhomax
c
      rhomax = exp(logrhomax)
c
c     Make the new rho
c
      do iz=1,ct_nz
          ct_rho_new(iz) = rhomax * 
     %         ( (ct_rho_orig(iz)/rhomax)**ct_pl + 1.d0 )**(1.d0/ct_pl)
          alpha(iz)      = ct_rho_new(iz) * ct_kappa(iz)
      enddo
c
c     Compute the optical depth
c
      tautot = integrate(ct_nz,ct_z,alpha)
c
c     Return tautot - taumax
c
      ct_routine = tautot - ct_taumax
      return
      end


c     --------------------------------------------------------------
c                     DO A SIMPLE NUMERICAL INTEGRATION
c     --------------------------------------------------------------
      function integrate(n,x,f)
      implicit none
      integer n,i
      doubleprecision x(n),f(n),integrate,int
      int=0.d0
      do i=2,n
          int=int+0.5d0*(f(i)+f(i-1))*(x(i)-x(i-1))
      enddo
      if(x(n).gt.x(1)) then
          integrate = int
      else
          integrate = -int
      endif
      return
      end


c     --------------------------------------------------------------
c                   HUNT: A NUMERICAL RECIPES ROUTINE
c     --------------------------------------------------------------
      SUBROUTINE hunt(xx,n,x,jlo)
      INTEGER jlo,n
      DOUBLEPRECISION x,xx(n)
      INTEGER inc,jhi,jm
      LOGICAL ascnd
      ascnd=xx(n).gt.xx(1)
      if(jlo.le.0.or.jlo.gt.n)then
        jlo=0
        jhi=n+1
        goto 3
      endif
      inc=1
      if(x.ge.xx(jlo).eqv.ascnd)then
1       jhi=jlo+inc
        if(jhi.gt.n)then
          jhi=n+1
        else if(x.ge.xx(jhi).eqv.ascnd)then
          jlo=jhi
          inc=inc+inc
          goto 1
        endif
      else
        jhi=jlo
2       jlo=jhi-inc
        if(jlo.lt.1)then
          jlo=0
        else if(x.lt.xx(jlo).eqv.ascnd)then
          jhi=jlo
          inc=inc+inc
          goto 2
        endif
      endif
3     if(jhi-jlo.eq.1)return
      jm=(jhi+jlo)/2
      if(x.gt.xx(jm).eqv.ascnd)then
        jlo=jm
      else
        jhi=jm
      endif
      goto 3
      END
C  (C) Copr. 1986-92 Numerical Recipes Software =v1.9"217..


c     --------------------------------------------------------------
c                  ZBRENT: A NUMERICAL RECIPES ROUTINE
c     --------------------------------------------------------------
      FUNCTION zbrent(func,x1,x2,tol)
      INTEGER ITMAX
      DOUBLEPRECISION zbrent,tol,x1,x2,func,EPS
      EXTERNAL func
      PARAMETER (ITMAX=100,EPS=3.e-8)
      INTEGER iter
      DOUBLEPRECISION a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm
      a=x1
      b=x2
      fa=func(a)
      fb=func(b)
      if((fa.gt.0..and.fb.gt.0.).or.(fa.lt.0..and.fb.lt.0.))stop
     *'root must be bracketed for zbrent'
      c=b
      fc=fb
      do 11 iter=1,ITMAX
        if((fb.gt.0..and.fc.gt.0.).or.(fb.lt.0..and.fc.lt.0.))then
          c=a
          fc=fa
          d=b-a
          e=d
        endif
        if(abs(fc).lt.abs(fb)) then
          a=b
          b=c
          c=a
          fa=fb
          fb=fc
          fc=fa
        endif
        tol1=2.*EPS*abs(b)+0.5*tol
        xm=.5*(c-b)
        if(abs(xm).le.tol1 .or. fb.eq.0.)then
          zbrent=b
          return
        endif
        if(abs(e).ge.tol1 .and. abs(fa).gt.abs(fb)) then
          s=fb/fa
          if(a.eq.c) then
            p=2.*xm*s
            q=1.-s
          else
            q=fa/fc
            r=fb/fc
            p=s*(2.*xm*q*(q-r)-(b-a)*(r-1.))
            q=(q-1.)*(r-1.)*(s-1.)
          endif
          if(p.gt.0.) q=-q
          p=abs(p)
          if(2.*p .lt. min(3.*xm*q-abs(tol1*q),abs(e*q))) then
            e=d
            d=p/q
          else
            d=xm
            e=d
          endif
        else
          d=xm
          e=d
        endif
        a=b
        fa=fb
        if(abs(d) .gt. tol1) then
          b=b+d
        else
          b=b+sign(tol1,xm)
        endif
        fb=func(b)
11    continue
      stop 'zbrent exceeding maximum iterations'
      zbrent=b
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software =v1.9"217..

c     --------------------------------------------------------------
c                      MAKE INDEXED FILENAME
c
c     This routine makes filenames of the type dusttemp_4.dat etc.
c     --------------------------------------------------------------
      subroutine make_indexed_filename(base,index,ext,filename)
      implicit none
      character*80 base
      integer index
      character*80 ext
      character*80 filename
      character*12 ch
c     
      if((index.lt.0).or.(index.ge.1000)) then
          write(*,*) 'ERROR in make_indexed_filename()'
          stop 729
      endif
      if(index.lt.10) then
          write(ch,11) index
 11       format(I1)
      elseif(index.lt.100) then
          write(ch,12) index
 12       format(I2)
      elseif(index.lt.1000) then
          write(ch,13) index
 13       format(I3)
      endif
      filename = base(1:len_trim(base))//
     %           ch(1:len_trim(ch))//
     %           ext(1:len_trim(ext))
c
      end


c     ==============================================================
c                 ROUTINES FOR PARLIST INPUT STYLE
c     ==============================================================

c     --------------------------------------------------------------
c            READ IN THE ENTIRE INPUT FILE INTO A STRING ARRAY
c     --------------------------------------------------------------
      subroutine read_input_file()
      implicit none
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      character*160 string
      integer iline,i
c
      do iline=1,INPUT_MAX_LEN
          read(1,'(A158)',end=10) string
          inpstring(iline) = string
          lineok(iline) = 0
      enddo
      write(*,*) 'Input file contains too many lines...'
      stop
 10   continue
      nrstring=iline-1
c      
      end



c     --------------------------------------------------------------
c         BASIC PARSING ROUTINE: SPLIT STRING INTO NAME AND VALUE
c
c     This routine interprets lines such as
c  
c       ; The following variable is useful:
c       var1   = 5.74d0  ; Here some comments
c
c     The first line is simply ignored (though marked as OK) and
c     the second line would be split into strname='var1' and
c     strvalue='5.74d0' with lenname being 4 and lenvalue being 6.
c     
c     --------------------------------------------------------------
      subroutine parse_name_value(string,strname,lenname,
     %                                   strvalue,lenvalue,icomm)
      implicit none
      character*160 string
      character*80 strname,strvalue
      integer lenname,lenvalue
      logical icomm
c
      integer ilen,i,istart,istop
c
      icomm    = .false.
      strname  = ''
      lenname  = 0
      strvalue = ''
      lenvalue = 0
      ilen     = len_trim(string)
      istart   = 1
      do i=1,ilen-1
          if(string(i:i).eq.' ') then
              istart=i+1
          else
              goto 20
          endif
      enddo
      icomm=.true.
      return
 20   continue
      if(string(istart:istart).eq.';') then
          icomm=.true.
          return
      endif
      if(istart.ge.ilen) return
      do i=istart+1,ilen-1
          if(string(i:i).eq.' ') goto 40
          if(string(i:i).eq.'=') goto 40
      enddo
      return
 40   continue
      istop=i-1
      lenname = istop-istart+1
      if(lenname.gt.80) then
          write(*,*) 'Variable name too long'
          stop
      endif
      strname = string(istart:istop)
      do i=istop+1,ilen-1
          if(string(i:i).eq.'=') goto 50
      enddo
      lenname=0
      strname=''
      lenvalue=0
      strvalue='' 
      return
 50   continue
      istart = i+1
      if(istart.gt.ilen) stop
      do i=istart,ilen
          if(string(i:i).eq.' ') then
              istart=i+1
          else
              goto 60
          endif
      enddo
 60   continue
      do i=istart+1,ilen
          if(string(i:i).eq.';') goto 80
          if(string(i:i).eq.' ') goto 80
      enddo
 80   continue
      istop = i-1
      lenvalue = istop-istart+1
      if(lenvalue.gt.80) then
          write(*,*) 'Value length too long'
          stop
      endif
      strvalue = string(istart:istop)
c
c      write(*,*) '##',strname(1:lenname),'##',strvalue(1:lenvalue),'##'
c     
      end


c     --------------------------------------------------------------
c                         STRING COMPARING
c     --------------------------------------------------------------
      function stringcompare(string1,string2,len)
      implicit none
      character*80 string1,string2
      integer len
      integer i
      logical stringcompare
c
      if(len.gt.80) stop
      if(len.le.0) then
          stringcompare=.false.
          return
      endif
      stringcompare=.true.
      do i=1,len
          if(string1(i:i).ne.string2(i:i)) stringcompare=.false.
      enddo
      return
      end


c     --------------------------------------------------------------
c                       PARSING ROUTINE FOR DOUBLE
c     --------------------------------------------------------------
      subroutine parse_input_double(name,value)
      implicit none
      character*80 name
      doubleprecision value
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      integer iline,ichar
      character*80 strname,strvalue
      integer lenname,lenvalue,lenn
      logical stringcompare,found,icomm
c
      found=.false.
      do ichar=2,80
          if(name(ichar:ichar).eq.'@') goto 30
      enddo
      write(*,*) 'INTERNAL ERROR: Keywords must end with @!!!'
      stop
 30   continue
      lenn=ichar-1
      do iline=1,nrstring
          call parse_name_value(inpstring(iline),
     %             strname,lenname,strvalue,lenvalue,icomm)
          if(icomm) then
              lineok(iline) = 1
          elseif(stringcompare(strname,name,lenn).and.
     %           (lenn.eq.lenname)) then
              if(found) then
                  write(*,*) 'Found one parameter more than once'
                  write(*,*) 'in the input file: ',strname(1:lenname)
                  write(*,*) '*** ABORTING ***'
                  stop
              endif
              read(strvalue(1:lenvalue),*) value
              found=.true.
              lineok(iline) = 1
          endif
      enddo
c
      end

c     --------------------------------------------------------------
c                       PARSING ROUTINE FOR INTEGER
c     --------------------------------------------------------------
      subroutine parse_input_integer(name,value)
      implicit none
      character*80 name
      integer value
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      integer iline,ichar
      character*80 strname,strvalue
      integer lenname,lenvalue,lenn
      logical stringcompare,found,icomm
c
      found=.false.
      do ichar=2,80
          if(name(ichar:ichar).eq.'@') goto 30
      enddo
      write(*,*) 'INTERNAL ERROR: Keywords must end with @!!!'
      stop
 30   continue
      lenn=ichar-1
      do iline=1,nrstring
          call parse_name_value(inpstring(iline),
     %             strname,lenname,strvalue,lenvalue,icomm)
          if(icomm) then
              lineok(iline) = 1
          elseif(stringcompare(strname,name,lenn).and.
     %           (lenn.eq.lenname)) then
              if(found) then
                  write(*,*) 'Found one parameter more than once'
                  write(*,*) 'in the input file: ',strname(1:lenname)
                  write(*,*) '*** ABORTING ***'
                  stop
              endif
              read(strvalue(1:lenvalue),*) value
              found=.true.
              lineok(iline) = 1
          endif
      enddo
c
      end

c     --------------------------------------------------------------
c                   CHECK IF ALL LINES ARE OK
c     --------------------------------------------------------------
      subroutine check_all_lines_ok(ierror)
      implicit none
      logical ierror
c
      common/parses/inpstring
      character*160 inpstring(INPUT_MAX_LEN)
      common/parsei/nrstring,lineok
      integer nrstring,lineok(INPUT_MAX_LEN)
c
      character*160 string
      integer iline,ilen
c
      ierror = .false.
      do iline=1,nrstring
          if(lineok(iline).eq.0) then
              write(*,*) 'ERROR in input file: variable unknown:'
              ilen = len_trim(inpstring(iline))
              string = inpstring(iline)
              write(*,*) string(1:ilen)
              ierror = .true.
          endif
      enddo
c
      end
