;----------------------------------------------------------
;          CALL RADICAL TO MAKE THE CIRCULAR IMAGE
;----------------------------------------------------------
pro makecircimage,incl=incl,ifreq=ifreq,if0=if0,if1=if1,lambda=lambda
AU  = 1.496d13       ; Astronomical Unit       [cm]
cc  = 2.9979d10      ; Light speed             [cm/s]
if n_elements(incl) eq 0 then incl=0.d0
angle = incl / 57.2957795131d0
if n_elements(lambda) ne 0 then begin
   nu = 1d4*cc/lambda
   q=findfile('frequency.inp',count=count)
   if count eq 0 then begin
      print,'Cannot find frequency.inp, dont know size...'
      stop
   endif
   openr,1,'frequency.inp'
   nf=0
   readf,1,nf
   freq=dblarr(nf)
   readf,1,freq
   close,1
   ifreq=-1
   for inu=0,nf-2 do begin
      if (freq[inu]-nu)*(freq[inu+1]-nu) le 0.d0 then ifreq=inu
   endfor
   if ifreq lt 0 then begin
      print,'Wavelength ',lambda,' out of range...'
      print,'Min and Max lambda = ',min(1d4*cc/freq),max(1d4*cc/freq)
      stop
   endif
   ; Find nearest wavelength
   if abs(freq[ifreq]-nu) gt abs(freq[ifreq+1]-nu) then ifreq=ifreq+1
endif
if n_elements(ifreq) ne 0 then begin
   if0 = ifreq
   if1 = ifreq
endif
if n_elements(if0) eq 0 or n_elements(if1) eq 0 then begin
   print,'ERROR: Must specify frequency'
   stop
endif
;
; Make the command file for RADICAL
;
openw,1,'command'
printf,1,'cirimageseq'
printf,1,'incl'
printf,1,strcompress(string(incl),/remove_all)
printf,1,'inu0'
printf,1,strcompress(string(if0),/remove_all)
printf,1,'inu1'
printf,1,strcompress(string(if1),/remove_all)
close,1
spawn,'nice radical'
end


;-----------------------------------------------------------------
;              READ THE CIRCULAR TELESCOPE IMAGE
;-----------------------------------------------------------------
function readcircimage,ext
nr=0
nphi=0
nf=0
if n_elements(ext) eq 0 then begin
   filename='circimage_sequence.dat'
endif else begin
   filename='circimage_'+strcompress(string(ext),/remove_all)+'.dat'
endelse
str=findfile(filename)
if(str(0) ne filename) then begin
    print,'Sorry, cannot find ',filename
    print,'Presumably radical exited without succes.'
    print,'See above for possible error messages of radical!'
    stop
endif
openr,1,filename
nf=0
readf,1,nf
freq=dblarr(nf)
readf,1,freq
nr=0
nphi=0
nf=0
readf,1,nr,nphi,nf
r=dblarr(nr)
ri=dblarr(nr+1)
phi=dblarr(nphi)
phii=dblarr(nphi+1)
readf,1,r
readf,1,ri
readf,1,phi
readf,1,phii
;r=[0.5*r[0],r]
image=dblarr(nphi,nr+1,nf)
readf,1,image
close,1
if nf gt 1 then begin
   image=transpose(image,[1,0,2])
endif else begin
   image=transpose(image,[1,0])
endelse
return,{r:r,ri:ri,nf:nf,phi:phi,phii:phii,image:image,freq:freq}
end


