c     ==============================================================
c                       VERTICAL STRUCTURE MODULE 
c     ==============================================================
#include "main.h"



c     --------------------------------------------------------------
c                  READ THE INPUT FILE FOR VERT STRUCT
c     --------------------------------------------------------------
      subroutine read_vstructinp()
      implicit none
c
#include "common_grid.h"
#include "common_dust.h"
#include "common_vstruct.h"
c
      integer iformat,nspec,ispec,idum
      logical fex
c
      inquire(file='vstruct.inp',exist=fex)
      if(fex) then 
          open(unit=1,file='vstruct.inp')
          read(1,*) iformat
          read(1,*) nspec
          if(nspec.ne.dust_nr_species) then
              write(*,*) 'ERROR: The nr of dust species specified in'
              write(*,*) '       vstruct.inp is not equal to that in'
              write(*,*) '       dustopac.inp'
              stop
          endif
          do ispec=1,nspec
              read(1,*) idum
              ivstr_dostruct(ispec) = idum
              if(idum.ne.0) then
                  write(*,*) '...For dust species ',ispec,
     %                 ' I will do vertical structure iteration.'
              endif
          enddo
          close(1)
      else
          write(*,*) 'Could not find file vstruct.inp. Therefore ',
     %         'doing vertical structure for all dust species.'
          do ispec=1,dust_nr_species
              ivstr_dostruct(ispec) = 1
          enddo
      endif
c
      end



c     --------------------------------------------------------------
c                     CALCULATE SURFACE DENSITY
c
c     NOTE: This is without sin(theta) and cos(theta) corrections
c     --------------------------------------------------------------
      subroutine calc_sigmadust()
      implicit none
c
#include "common_grid.h"
#include "common_dust.h"
#include "common_boundary.h"
#include "common_montecarlo.h"
#include "common_vstruct.h"
c
      doubleprecision dum
      integer ir,it,ispec,nt
      doubleprecision pi
      parameter(pi=3.1415926535897932385d0)
c
c     Calculate the surface density
c     
      nt=irsi_frsizey/2
      do ir=1,irsi_frsizex
          do ispec=1,dust_nr_species
              dum = 0.d0
c              do it=2,nt
c                  dum = dum + 0.5 * ( dust_rho(ispec,it,ir) + 
c     %                  dust_rho(ispec,it-1,ir) ) * rsi_x_c(ir,1) *
c     %                   abs( rsi_x_c(it,2) - rsi_x_c(it-1,2) )
c              enddo
              do it=1,nt
                  dum = dum + dust_rho(ispec,it,ir) * rsi_x_c(ir,1) *
     %                   abs( rsi_x_i(it,2) - rsi_x_i(it+1,2) )
              enddo
              vstr_sigmadust(ir,ispec) = 2*dum
          enddo
      enddo
c
c     Calculate the dust mass in each dust species
c
      vstr_massdusttot = 0.d0
      do ispec=1,dust_nr_species
          vstr_massdust(ispec) = 0.d0
c          do ir=2,irsi_frsizex
c              vstr_massdust(ispec) = vstr_massdust(ispec) +
c     %              pi*(rsi_x_c(ir,1)**2 - rsi_x_c(ir-1,1)**2)*
c     %              0.5*(vstr_sigmadust(ir-1,ispec)+
c     %                   vstr_sigmadust(ir,ispec))
c          enddo
          do ir=1,irsi_frsizex
              vstr_massdust(ispec) = vstr_massdust(ispec) +
     %              pi*(rsi_x_i(ir+1,1)**2 - rsi_x_i(ir,1)**2)*
     %                   vstr_sigmadust(ir,ispec)
          enddo
          vstr_massdusttot = vstr_massdusttot + vstr_massdust(ispec)
      enddo
c
      end




c     --------------------------------------------------------------
c                 INTEGRATE THE VERTICAL STRUCTURE 
c
c     The equation to solve is:
c
c      dlg(rho)     / mu mp G M  z    dlg(T) \
c      -------- = - | ---------  -  + ------ |
c        dz         \   k R^3    T      dz   /
c     
c     ARGUMENTS:
c       ivstrt    Which dust species represents T_gas?
c
c     RETURNS:
c       error     The max difference in rho(z=0) before/after this 
c                 calculation. Note that I do not use the whole
c                 solution because in the surface layers can have
c                 very strong fluctuations in rho for very tiny
c                 fluctuations in H_p.
c       iwarn     If 1, then the resolution of the theta-grid near
c                 the midplane is not high enough, or in other words
c                 most of the solution lies in the lower 2 grid 
c                 points, which is inaccurate.
c
c     --------------------------------------------------------------
      subroutine vertstruct_integrate(tdust,ivstrt,error,iwarn)
      implicit none
      doubleprecision error
      integer iwarn,ivstrt
      doubleprecision tdust(DUST_SIZE_MAX,DUST_SPECIES_MAX,
     %                      FRSIZE_Y_SMALL,FRSIZE_X)
c
      doubleprecision GG,mp,me,hh,kk,ee,cc,st,ss,aa
      parameter(GG  = 6.672d-8)      ! Gravitational constant
      parameter(mp  = 1.6726d-24)    ! Mass of proton          [g]
      parameter(me  = 9.1095d-28)    ! Mass of electron        [g]
      parameter(kk  = 1.3807d-16)    ! Bolzmann's constant     [erg/K]
      parameter(hh  = 6.6262d-27)    ! Planck's constant       [erg.s]
      parameter(ee  = 4.8032d-10)    ! Unit charge             
      parameter(cc  = 2.9979d10)     ! Light speed             [cm/s]
      parameter(st  = 6.6524d-25)    ! Thompson cross-section  [cm^2]
      parameter(ss  = 5.6703d-5)     ! Stefan-Boltzmann const  [erg/cm^2/K^4/s]
      parameter(aa  = 7.5657d-15)    ! 4 ss / cc               [erg/cm^3/K^4]
      doubleprecision pi
      parameter(pi  = 3.1415926535897932385d0)
c
#include "common_grid.h"
#include "common_dust.h"
#include "common_vstruct.h"
#include "common_boundary.h"
c
      doubleprecision mugas,gravc,r,errloc,dlgt,dz,grv,sig0
      doubleprecision rho0(FRSIZE_Y),z(FRSIZE_Y),zi(FRSIZE_Y+1)
      integer nr,nt,ir,it,ispec,number_invalid
c
c     Check something
c
      if(ivstrt.gt.dust_nr_species) then
          write(*,*) 'ERROR: ivstrt greater than nr of species'
          stop
      endif
c
c     Some constants
c
      nr = irsi_frsizex
      nt = irsi_frsizey/2
      error = 0.d0
c
c     Check validity of all temperatures
c
#     ifdef CHECK_NUMBERS
      do ispec=1,dust_nr_species 
          do ir=1,nr
              do  it=1,nt
                  if(number_invalid(tdust(1,ispec,it,ir)).ne.0) then
                      write(*,*) 'ERROR: Invalid temperature found'
                      write(*,*) ir,it,ispec,tdust(1,ispec,it,ir)
                      stop 5927
                  endif
                  if(tdust(1,ispec,it,ir).le.0.d0) then
                      write(*,*) 'ERROR: Zero temperature found'
                      write(*,*) '   while preparing to do vertical'
                      write(*,*) '   structure integration.'
                      write(*,*) ir,it,ispec,tdust(1,ispec,it,ir)
                      stop 5928
                  endif
              enddo
          enddo
      enddo
#     endif
c
c     We take mugas fixed:
c
      mugas = 2.3
c
c     Make a loop over dust species
c
      do ispec=1,dust_nr_species 
c
c         Check if we need to do this species
c
          if(ivstr_dostruct(ispec).ne.0) then
c
c             Make a loop over r
c
              do ir=1,nr
c
c                 Get the radius
c
                  r  = rsi_x_c(ir,1)
c
c                 Make the z-grid
c
                  do it=1,nt
                      z(it) = r * (0.5d0*pi-rsi_x_c(it,2))
                  enddo
                  do it=1,nt+1
                      zi(it) = r * (0.5d0*pi-rsi_x_i(it,2))
                  enddo
c
c                 Compute some constants
c
                  gravc = (GG*radbnd_mstar/r**3)*mugas*mp/kk
c
c                 Now perform the integration 
c
                  if(tdust(1,ivstrt,nt,ir).le.0.d0) then
                      write(*,*) 'ERROR in vertical structure ',
     %                     'integration: temp le 0 at ',ir,nt,
     %                     ', ivstrt=',ivstrt
                      stop
                  endif
                  rho0(nt) = 1.d0
                  do it=nt,2,-1
                      if(tdust(1,ivstrt,it-1,ir).le.0.d0) then
                          write(*,*) 'ERROR in vertical structure ',
     %                            'integration: temp le 0 at ',ir,it-1,
     %                            ', ivstrt=',ivstrt
                          stop
                      endif
                      dz   = (z(it-1)-z(it))
                      dlgt = (log(tdust(1,ivstrt,it-1,ir))-
     %                        log(tdust(1,ivstrt,it,ir)))/dz
                      grv  = 0.5 * gravc * 
     %                       ( z(it-1)/tdust(1,ivstrt,it-1,ir) +
     %                         z(it)/tdust(1,ivstrt,it,ir) )
                      rho0(it-1) = rho0(it) * exp ( -dz*(grv+dlgt) )
                  enddo
c
c                 Now find the Sigma for this rho
c
                  sig0 = 0.d0
c                  do it=nt,2,-1
c                      dz   = (z(it-1)-z(it))
c                      sig0 = sig0 + 0.5*(rho0(it-1)+rho0(it))*dz
c                  enddo
                  do it=nt,1,-1
                      dz   = abs(zi(it)-zi(it+1))
                      sig0 = sig0 + rho0(it)*dz
                  enddo
                  sig0 = sig0*2 ! Both disk sides
c
c                 Normalize the rho 
c
                  do it=nt,1,-1
                      rho0(it) = rho0(it) * 
     %                    vstr_sigmadust(ir,ispec) / sig0
                      if(rho0(it).lt.1.d-90) then
                          rho0(it) = 1.d-90
                      endif
                  enddo
c
c                 Compute error at midplane
c
                  errloc = 2*abs(rho0(nt)-dust_rho(ispec,nt,ir)) / 
     %                          (rho0(nt)+dust_rho(ispec,nt,ir))
                  if(errloc.gt.error) then
                      error = errloc
                  endif
c
c                 Insert back into the dust configuration
c
                  do it=nt,1,-1
                      dust_rho(ispec,it,ir) = rho0(it)
                  enddo                  
c
c                 Check
c
                  if(abs(rho0(nt)-rho0(nt-1))/
     %                  (rho0(nt)+rho0(nt-1)).gt.0.3) then
                      iwarn=1
                  else 
                      iwarn=0
                  endif
              enddo
          endif
      enddo
c
      end
